@@ -1,5 +1,6 @@
 import numpy as np
 import math, copy, cctk, os, re, logging, time
+import fasteners
 
 import h5py
 import presto
@@ -20,33 +21,79 @@ class Trajectory():
         atomic_numbers (cctk.OneIndexedArray): list of atomic numbers
         masses (cctk.OneIndexedArray): list of masses
 
-        calculator (presto.Calculator):
-        integrator (presto.Integrator):
+        calculator (presto.calculators.Calculator):
+        integrator (presto.integrators.Integrator):
+        reporters (list of presto.reporters.Reporter):
+        checks (list of presto.checks.Check):
 
         finished (bool):
         forwards (bool):
 
         checkpoint_filename (str):
+        checkpoint_interval (int):
+        lock (fasteners.InterProcessLock): lock object
+        save_interval (int): how many frames to save
     """
 
-    def __init__(self, calculator=None, integrator=None, timestep=None, atomic_numbers=None, high_atoms=None, forwards=True, checkpoint_filename=None, stop_time=None, **kwargs):
+    def __init__(
+        self,
+        calculator=None,
+        integrator=None,
+        reporters=list(),
+        checks=list(),
+        timestep=None,
+        atomic_numbers=None,
+        high_atoms=None,
+        forwards=True,
+        checkpoint_filename=None,
+        checkpoint_interval=10,
+        stop_time=None,
+        save_interval=1,
+        load_frames="all", # or ``first`` or ``last`` or a slice
+        **kwargs
+    ):
+
+        # do this first!
+        if timestep is not None:
+            assert timestep > 0, "can't have timestep ≤ 0!"
+            self.timestep = float(timestep)
+
         if checkpoint_filename is not None:
             assert isinstance(checkpoint_filename, str), "need string for file"
         self.checkpoint_filename = checkpoint_filename
+
+        assert isinstance(checkpoint_interval, int) and checkpoint_interval > 0, "checkpoint_interval must be positive integer"
+        self.checkpoint_interval = checkpoint_interval
+
+        self.lock = None
+        self.initialize_lock()
+        self.frames = list()
+
         if self.has_checkpoint():
-            if "load_frames" in kwargs:
-                self.load_from_checkpoint(kwargs["load_frames"])
+            if load_frames=="all":
+                self.load_from_checkpoint(slice(None))
+            elif load_frames=="first":
+                self.load_from_checkpoint(slice(1,None,None))
+            elif load_frames=="last":
+                self.load_from_checkpoint(slice(-1,None,None))
             else:
-                self.load_from_checkpoint()
+                assert isinstance(load_frames, slice), "load_frames must be ``all``, ``first``, ``last``, or slice"
+                self.load_from_checkpoint(load_frames)
 
         if calculator is not None:
             assert isinstance(calculator, presto.calculators.Calculator), "need a valid calculator!"
+        self.calculator = calculator
+
         if integrator is not None:
             assert isinstance(integrator, presto.integrators.Integrator), "need a valid integrator!"
-
-        self.calculator = calculator
         self.integrator = integrator
 
+        assert all([isinstance(c, presto.checks.Check) for c in checks])
+        self.checks = checks
+
+        assert all([isinstance(r, presto.reporters.Reporter) for r in reporters])
+        self.reporters = reporters
+
         if atomic_numbers is not None:
             assert isinstance(atomic_numbers, cctk.OneIndexedArray), "atomic numbers must be cctk 1-indexed array!"
             self.atomic_numbers = atomic_numbers
@@ -73,10 +120,6 @@ class Trajectory():
             # assume all atoms are active
             self.set_inactive_atoms(np.ndarray([]))
 
-        if timestep is not None:
-            assert timestep > 0, "can't have timestep ≤ 0!"
-            self.timestep = float(timestep)
-
         if not hasattr(self, "masses"):
             self.masses = cctk.OneIndexedArray([float(cctk.helper_functions.draw_isotopologue(z)) for z in atomic_numbers])
 
@@ -94,6 +137,10 @@ class Trajectory():
             assert stop_time > 0, "stop_time needs to be positive!"
             self.stop_time = stop_time
 
+        assert isinstance(save_interval, int), "save_interval needs to be positive"
+        assert save_interval > 0, "save_interval needs to be positive"
+        self.save_interval = save_interval
+
     def __str__(self):
         return f"Trajectory({len(self.frames)} frames)"
 
@@ -112,40 +159,38 @@ class Trajectory():
         active_atoms = np.array(active_atoms)
         self.active_atoms = active_atoms
 
-    def run(self, checkpoint_interval=10, keep_all=False, time=None, **kwargs):
+    def run(self, keep_all=False, time=None, **kwargs):
         """
         Run the trajectory.
 
         Args:
-            checkpoint_interval (int): interval at which to save (in frames, not fs)
             keep_all (bool): whether or not to keep all frames in memory
-            num (float): total time to run for -- default is None, implying trajectory should be run until finished
+            time (float): total time to run for -- default is None, implying trajectory should be run until finished
         """
-        assert isinstance(self.calculator, presto.calculators.Calculator), "need a valid calculator!"
-        assert isinstance(self.integrator, presto.integrators.Integrator), "need a valid integrator!"
         if self.checkpoint_filename is None:
             if "checkpoint_filename" in kwargs:
                 self.checkpoint_filename = kwargs["checkpoint_filename"]
             else:
                 raise ValueError("no checkpoint filename given!")
 
-        if self.has_checkpoint():
-            self.load_from_checkpoint()
-
-        if len(self.frames) == 0:
-            self.initialize(**kwargs)
-        else:
-            self.frames = [self.frames[-1]] # we don't load all frames while running to keep things fast
+        self.load_from_checkpoint(slice(-1, None, None))
+        assert len(self.frames) == 1, "Wrong number of frames - do you need to call trajectory.initialize()?"
 
         if self.finished:
             logger.info("Trajectory already finished!")
-            return self
         else:
-            logger.info("Propagating trajectory.")
-            self.propagate(checkpoint_interval, keep_all=keep_all, time=time)
-            logger.info("Trajectory finished!")
-            self.save(keep_all=keep_all)
-            return self
+            # initialize runtime controller
+            controller = presto.controller.Controller(self, **kwargs)
+            try:
+                controller.run(runtime=time)
+            except Exception as e:
+                raise ValueError(f"Trajectory run terminated prematurely due to error: {e}")
+
+        if keep_all:
+            self.load_from_checkpoint()
+            assert self.frames[0].time == 0, "missing first frame despite keep_all being True!"
+
+        return self
 
     def initialize(self):
         """
@@ -153,20 +198,7 @@ class Trajectory():
         """
         pass
 
-    def propagate(self, checkpoint_interval, keep_all=False, time=None):
-        """
-        Runs trajectories, checking for completion and saving as necessary.
-
-        Will call ``self.save()`` every ``checkpoint_interval`` frames.
-
-        Args:
-            checkpoint_interval (int): interval at which to save (in frames, not fs)
-            keep_all (bool): whether or not to keep all frames in memory
-            time (float): total time to run for -- default is None, implying trajectory should be run until finished
-        """
-        pass
-
-    def has_checkpoint(self, max_wait=100):
+    def has_checkpoint(self):
         if self.checkpoint_filename is None:
             return False
         if os.path.exists(self.checkpoint_filename):
@@ -174,38 +206,7 @@ class Trajectory():
         else:
             return False
 
-        #### TODO - prevent lockfile collisions here and in num_frames
-
-#            lockfile = f"{self.checkpoint_filename}.lock"
-#
-#            while os.path.exists(lockfile):
-#                max_wait += -1
-#                time.sleep(1)
-#
-#                if max_wait < 0:
-#                    raise ValueError(f"can't get to file {self.checkpoint_filename}; max_wait exceeded!")
-#
-#            # create new lockfile
-#            with open(lockfile, "w") as lf:
-#                pass
-#
-#            status = False
-#            with h5py.File(self.checkpoint_filename, "r+") as h5:
-#                all_energies = h5.get("all_energies")
-#                if all_energies is None:
-#                    status = False
-#                elif len(all_energies) > 0:
-#                    status = True
-#                else:
-#                    status = False
-#
-#            os.remove(lockfile)
-#            return status
-#
-#        else:
-#            return False
-
-    def load_from_checkpoint(self, frames=slice(None), max_wait=100):
+    def load_from_checkpoint(self, frames=slice(None)):
         """
         Loads frames from ``self.checkpoint_filename``.
 
@@ -215,19 +216,11 @@ class Trajectory():
         Returns:
             nothing
         """
-        assert self.has_checkpoint(), "can't load without checkpoint file"
+        if not self.has_checkpoint():
+            return # nothing to load!
 
-        lockfile = f"{self.checkpoint_filename}.lock"
-        while os.path.exists(lockfile):
-            max_wait += -1
-            time.sleep(1)
-
-            if max_wait < 0:
-                raise ValueError(f"can't get to file {self.checkpoint_filename}; max_wait exceeded!")
-
-        # create new lockfile
-        with open(lockfile, "w") as lf:
-            pass
+        self.initialize_lock()
+        self.lock.acquire()
 
         with h5py.File(self.checkpoint_filename, "r") as h5:
             atomic_numbers = h5.attrs["atomic_numbers"]
@@ -239,29 +232,43 @@ class Trajectory():
             self.finished = h5.attrs['finished']
             self.forwards = h5.attrs['forwards']
 
-            all_energies = h5.get("all_energies")[frames]
-            all_positions = h5.get("all_positions")[frames]
-            all_velocities= h5.get("all_velocities")[frames]
-            all_accels = h5.get("all_accelerations")[frames]
-            temperatures = h5.get("bath_temperatures")[frames]
-
-            assert len(all_positions) == len(all_energies)
-            assert len(all_velocities) == len(all_energies)
-            assert len(all_accels) == len(all_energies)
-
             self.frames = []
-            for i, e in enumerate(all_energies):
-                self.frames.append(presto.frame.Frame(
-                    self,
-                    all_positions[i].view(cctk.OneIndexedArray),
-                    all_velocities[i].view(cctk.OneIndexedArray),
-                    all_accels[i].view(cctk.OneIndexedArray),
-                    energy=e,
-                    bath_temperature=temperatures[i]
-                ))
+            if len(h5.get("all_energies")):
+                all_energies = h5.get("all_energies")[frames]
+                all_positions = h5.get("all_positions")[frames]
+                all_velocities= h5.get("all_velocities")[frames]
+                all_accels = h5.get("all_accelerations")[frames]
+                temperatures = h5.get("bath_temperatures")[frames]
+
+                # v0.2.2 - provisionally removing this
+#                all_times = None
+#                try:
+                all_times = h5.get("all_times")[frames]
+#                except Exception as e:
+#                    all_times = np.arange(0, self.timestep*len(all_energies)*self.save_interval, self.timestep*self.save_interval)
+#                    # this was added recently, so may be some backwards compatibility issues.
+#                    pass
+
+                if isinstance(all_energies, np.ndarray):
+                    assert len(all_positions) == len(all_energies)
+                    assert len(all_velocities) == len(all_energies)
+                    assert len(all_accels) == len(all_energies)
+                    assert len(all_times) == len(all_energies)
+
+                for i, t in enumerate(all_times):
+                    self.frames.append(presto.frame.Frame(
+                        self,
+                        all_positions[i].view(cctk.OneIndexedArray),
+                        all_velocities[i].view(cctk.OneIndexedArray),
+                        all_accels[i].view(cctk.OneIndexedArray),
+                        energy=all_energies[i],
+                        bath_temperature=temperatures[i],
+                        time=all_times[i],
+                    ))
+
         logger.info(f"Loaded trajectory from checkpoint file {self.checkpoint_filename} -- {len(self.frames)} frames read.")
 
-        os.remove(lockfile)
+        self.lock.release()
         return
 
     def num_frames(self):
@@ -273,63 +280,76 @@ class Trajectory():
         else:
             return len(self.frames)
 
-    def save(self, keep_all=False, max_wait=100):
+    def save(self, keep_all=False):
         if self.checkpoint_filename is None:
             raise ValueError("can't save without checkpoint filename")
+        self.initialize_lock()
+        self.lock.acquire()
 
-        lockfile = f"{self.checkpoint_filename}.lock"
-        while os.path.exists(lockfile):
-            max_wait += -1
-            time.sleep(1)
-
-            if max_wait < 0:
-                raise ValueError(f"can't get to file {self.checkpoint_filename}; max_wait exceeded!")
-
-        # create new lockfile
-        with open(lockfile, "w") as lf:
-            pass
-
+        last_run_time = self.frames[-1].time
         if self.has_checkpoint():
             with h5py.File(self.checkpoint_filename, "r+") as h5:
                 n_atoms = len(self.atomic_numbers)
                 h5.attrs['finished'] = self.finished
-                all_energies = h5.get("all_energies")
 
+                all_energies = h5.get("all_energies")
                 old_n_frames = len(all_energies)
-                new_n_frames = len(self.frames) - 1
+                # v0.2.2 - provisionally removing this
+#                if "all_times" not in h5:
+#                    # time is a new column, so old checkpoints may not have it.
+#                    old_times = np.arange(0, self.timestep, self.timestep*old_n_frames)
+#                    h5.create_dataset("all_times", data=old_times, maxshape=(None,), compression="gzip", compression_opts=9)
+
+                all_times = h5.get("all_times")
+                last_saved_time = all_times[-1]
+                new_n_frames = int((last_run_time - last_saved_time) / (self.timestep*self.save_interval))
                 now_n_frames = new_n_frames + old_n_frames
 
                 if new_n_frames == 0:
-                    os.remove(lockfile)
+                    self.lock.release()
                     return
                 assert new_n_frames > 0, f"we can't write negative frames ({old_n_frames} previously in {self.checkpoint_filename}, but now only {now_n_frames})"
 
-                new_energies = np.asarray([frame.energy for frame in self.frames[-new_n_frames-1:]])
-                all_energies.resize((now_n_frames,))
-                all_energies[-new_n_frames-1:] = new_energies
+                frames_to_add = list()
+                # there is probably a more elegant way to handle this, but this seems robust at least
+                for frame in self.frames[-(new_n_frames*self.save_interval)-1:]:
+                    if frame.time <= last_saved_time:
+                        continue
+                    if frame.time % (self.timestep * self.save_interval) == 0:
+                        frames_to_add.append(frame)
+                assert new_n_frames == len(frames_to_add), "pernicious math error in frame numbers!"
 
-                new_positions = np.stack([frame.positions for frame in self.frames[-new_n_frames:]])
+                new_times = np.asarray([frame.time for frame in frames_to_add])
+                all_times.resize((now_n_frames,))
+                all_times[-new_n_frames:] = new_times
+
+                all_energies = h5.get("all_energies")
+                new_energies = np.asarray([frame.energy for frame in frames_to_add])
+                all_energies.resize((now_n_frames,))
+                all_energies[-new_n_frames:] = new_energies
+
+                new_positions = np.stack([frame.positions for frame in frames_to_add])
                 all_positions = h5.get("all_positions")
                 all_positions.resize((now_n_frames,n_atoms,3))
                 all_positions[-new_n_frames:] = new_positions
 
-                new_velocities= np.stack([frame.velocities for frame in self.frames[-new_n_frames:]])
+                new_velocities= np.stack([frame.velocities for frame in frames_to_add])
                 all_velocities = h5.get("all_velocities")
                 all_velocities.resize((now_n_frames,n_atoms,3))
                 all_velocities[-new_n_frames:] = new_velocities
 
-                new_accels = np.stack([frame.accelerations for frame in self.frames[-new_n_frames:]])
+                new_accels = np.stack([frame.accelerations for frame in frames_to_add])
                 all_accels = h5.get("all_accelerations")
                 all_accels.resize((now_n_frames,n_atoms,3))
                 all_accels[-new_n_frames:] = new_accels
 
-                new_temps = np.stack([frame.bath_temperature for frame in self.frames[-new_n_frames:]])
+                new_temps = np.stack([frame.bath_temperature for frame in frames_to_add])
                 all_temps = h5.get("bath_temperatures")
                 all_temps.resize((now_n_frames,))
                 all_temps[-new_n_frames:] = new_temps
-                logger.info(f"Saving trajectory to existing checkpoint file {self.checkpoint_filename} ({new_n_frames} frames added; {now_n_frames} in total)")
+
+            logger.info(f"Saving to existing checkpoint file {self.checkpoint_filename} ({new_n_frames} frames added; {last_run_time:.1f}/{self.stop_time:.1f} fs run in total)")
         else:
-            logger.info(f"Saving trajectory to new checkpoint file {self.checkpoint_filename} ({len(self.frames)} frames)")
             with h5py.File(self.checkpoint_filename, "w") as h5:
                 h5.attrs['atomic_numbers'] = self.atomic_numbers.view(np.ndarray)
                 h5.attrs['masses'] = self.masses.view(np.ndarray)
@@ -338,27 +358,31 @@ class Trajectory():
 
                 n_atoms = len(self.atomic_numbers)
 
-                energies = np.asarray([frame.energy for frame in self.frames])
-                h5.create_dataset("all_energies", data=energies, maxshape=(None,),
-                            compression="gzip", compression_opts=9)
+                frames_to_add = list()
+                for frame in self.frames:
+                    if frame.time % (self.timestep * self.save_interval) == 0:
+                        frames_to_add.append(frame)
 
-                all_positions = np.stack([frame.positions for frame in self.frames])
-                h5.create_dataset("all_positions", data=all_positions, maxshape=(None,n_atoms,3),
-                                compression="gzip", compression_opts=9)
+                energies = np.asarray([frame.energy for frame in frames_to_add])
+                h5.create_dataset("all_energies", data=energies, maxshape=(None,), compression="gzip", compression_opts=9)
 
-                all_velocities = np.stack([frame.velocities for frame in self.frames])
-                h5.create_dataset("all_velocities", data=all_velocities, maxshape=(None,n_atoms,3),
-                                compression="gzip", compression_opts=9)
+                times = np.asarray([frame.time for frame in frames_to_add])
+                h5.create_dataset("all_times", data=times, maxshape=(None,), compression="gzip", compression_opts=9)
 
-                all_accels= np.stack([frame.accelerations for frame in self.frames])
-                h5.create_dataset("all_accelerations", data=all_accels, maxshape=(None,n_atoms,3),
-                                compression="gzip", compression_opts=9)
+                all_positions = np.stack([frame.positions for frame in frames_to_add])
+                h5.create_dataset("all_positions", data=all_positions, maxshape=(None,n_atoms,3), compression="gzip", compression_opts=9)
 
-                temps = np.asarray([frame.bath_temperature for frame in self.frames])
-                h5.create_dataset("bath_temperatures", data=temps, maxshape=(None,),
-                            compression="gzip", compression_opts=9)
+                all_velocities = np.stack([frame.velocities for frame in frames_to_add])
+                h5.create_dataset("all_velocities", data=all_velocities, maxshape=(None,n_atoms,3), compression="gzip", compression_opts=9)
 
-        os.remove(lockfile)
+                all_accels= np.stack([frame.accelerations for frame in frames_to_add])
+                h5.create_dataset("all_accelerations", data=all_accels, maxshape=(None,n_atoms,3), compression="gzip", compression_opts=9)
+
+                temps = np.asarray([frame.bath_temperature for frame in frames_to_add])
+                h5.create_dataset("bath_temperatures", data=temps, maxshape=(None,), compression="gzip", compression_opts=9)
+
+            logger.info(f"Saving to new checkpoint file {self.checkpoint_filename} ({len(frames_to_add)} frames added; {last_run_time:.1f}/{self.stop_time:.1f} fs run in total)")
+        self.lock.release()
 
         # lower memory usage
         if keep_all:
@@ -366,11 +390,34 @@ class Trajectory():
         else:
             self.frames = [self.frames[-1]]
 
-    def write_movie(self, filename, idxs="high"):
-        if idxs == "high":
-            idxs = self.high_atoms
-        elif idxs == "all":
-            idxs = None
+    def write_movie(self, filename, solvents="all", idxs=None):
+        """
+        Write a movie to a trajectory file. Detects trajectory type automatically from file extension.
+
+        Supported file formats: ``.pdb``, ``.mol2``, ``.xyz``/``.molden``
+
+        Args:
+            filename (str): path where movie will be written
+            solvents (int): number of solvent molecules to include (closest included first). can also be ``none`` or ``all``.
+            idxs (list of int): indices of atoms to include. will override ``solvents`` if present.
+        """
+
+        # what do we make a movie of?
+        if idxs is not None:
+            assert isinstance(idxs, list), "idxs must be list of indices or ``None``!"
+        else:
+            if isinstance(solvents, str):
+                if solvents == "high":
+                    idxs = self.high_atoms
+                elif solvents == "all":
+                    idxs = None
+                else:
+                    raise ValueError(f"unknown solvents keyword {solvents} -- must be 'high' or 'all'")
+            elif isinstance(solvents, int):
+                molecule = self.frames[0].molecule().assign_connectivity()
+                idxs = molecule.limit_solvent_shell(num_solvents=solvents, return_idxs=True)
+            else:
+                raise ValueError("``solvents`` must be int, 'high', or 'all'!")
 
         ensemble = self.as_ensemble(idxs)
         logger.info("Writing trajectory to {filename}")
@@ -378,9 +425,10 @@ class Trajectory():
             cctk.PDBFile.write_ensemble_to_trajectory(filename, ensemble)
         elif re.search("mol2$", filename):
             #### connectivity matters
-            for molecule in ensemble.molecules:
-                molecule.assign_connectivity()
+            ensemble.assign_connectivity()
             cctk.MOL2File.write_ensemble_to_file(filename, ensemble)
+        elif re.search("molden$", filename) or re.search("xyz$", filename):
+            cctk.XYZFile.write_ensemble_to_file(filename, ensemble)
         else:
             raise ValueError(f"error writing {filename}: this filetype isn't currently supported!")
 
@@ -390,71 +438,6 @@ class Trajectory():
             ensemble.add_molecule(frame.molecule(idxs), {"bath_temperature": frame.bath_temperature, "energy": frame.energy})
         return ensemble
 
-    def spawn_reaction_trajectory(self, termination_function, stop_time, f_filename=None, r_filename=None):
-        """
-        Args:
-            termination_function:
-            max_time:
-
-        Returns:
-            forward ReactionTrajectory
-            reverse ReactionTrajectory
-        """
-        if os.path.exists(f_filename) and os.path.exists(r_filename):
-            logger.info(f"Trajectories to spawn already exist!")
-
-            f_traj = ReactionTrajectory(calculator=self.calculator, integrator=self.integrator, checkpoint_filename=f_filename)
-            f_traj.termination_function = termination_function
-            f_traj.stop_time = stop_time
-
-            r_traj = ReactionTrajectory(calculator=self.calculator, integrator=self.integrator, checkpoint_filename=r_filename, forwards=False)
-            r_traj.termination_function = termination_function
-            r_traj.stop_time = stop_time
-
-            return f_traj, r_traj
-
-        # add random velocity to previously inactive atoms
-        frame = self.frames[frame_idx]
-
-        random_gaussian = np.random.normal(size=frame.positions.shape).view(cctk.OneIndexedArray)
-        random_gaussian[frame.active_mask()] = 0
-        new_velocities = random_gaussian * np.sqrt(frame.bath_temperature * presto.constants.BOLTZMANN_CONSTANT / self.masses.reshape(-1,1))
-
-        f_traj = ReactionTrajectory(
-            timestep = self.timestep,
-            atomic_numbers = self.atomic_numbers,
-            high_atoms = self.high_atoms,
-            active_atoms = np.array(list(range(1, len(self.atomic_numbers)+1))), # all atoms are active now
-            calculator = self.calculator,
-            integrator = self.integrator,
-            checkpoint_filename = f_filename,
-        )
-
-        f_traj.termination_function = termination_function
-        f_traj.stop_time = stop_time
-        f_traj.initialize(frame=frame, new_velocities=new_velocities)
-
-        r_traj = ReactionTrajectory(
-            timestep = self.timestep,
-            atomic_numbers = self.atomic_numbers,
-            high_atoms = self.high_atoms,
-            active_atoms = np.array(list(range(1, len(self.atomic_numbers)+1))), # all atoms are active now
-            calculator = self.calculator,
-            integrator = self.integrator,
-            checkpoint_filename = r_filename,
-        )
-
-        r_traj.forwards = False
-        r_traj.termination_function = termination_function
-        r_traj.stop_time = stop_time
-        r_traj.initialize(frame=frame, new_velocities=new_velocities)
-
-        assert os.path.exists(f_filename), f"didn't save to {f_filename} successfully"
-        assert os.path.exists(r_filename), f"didn't save to {r_filename} successfully"
-        logger.info(f"Two new reaction trajectories spawned and saved to {f_filename} and {r_filename}")
-
-        return f_traj, r_traj
-
     @classmethod
     def new_from_checkpoint(cls, checkpoint, frame):
         """
@@ -475,6 +458,25 @@ class Trajectory():
         assert len(new_traj.frames) == 1, "got too many frames!"
         return new_traj
 
+    def initialize_lock(self):
+        """
+        Create hidden lockfile to accompany ``.chk`` file.
+        """
+        if self.checkpoint_filename is None:
+            return
+
+        if self.lock is None:
+            lockfile = None
+            if "/" in self.checkpoint_filename:
+                lockfile = f"{self.checkpoint_filename}.lock"[::-1].replace("/", "./", 1)[::-1]
+            else:
+                lockfile = f".{self.checkpoint_filename}.lock"
+            self.lock = fasteners.InterProcessLock(lockfile)
+
+    def last_time_run(self):
+        """ Get last finished time """
+        return self.frames[-1].time
+
 class ReactionTrajectory(Trajectory):
     """
     Attributes:
@@ -522,7 +524,7 @@ class ReactionTrajectory(Trajectory):
         """
         logger.info("Initializing new reaction trajectory...")
         if self.has_checkpoint():
-            self.load_from_checkpoint()[-1]
+            self.load_from_checkpoint(slice(-1,None,None))
             return
 
         if frame is not None:
@@ -544,7 +546,7 @@ class ReactionTrajectory(Trajectory):
             assert isinstance(accelerations, cctk.OneIndexedArray)
             assert isinstance(bath_temp, (float, int, np.integer))
 
-        new_frame = presto.frame.Frame(self, positions, velocities, accelerations, bath_temp)
+        new_frame = presto.frame.Frame(self, positions, velocities, accelerations, bath_temperature=bath_temp, time=0.0)
 
         if new_velocities is None:
             random_gaussian = np.random.normal(size=positions.shape).view(cctk.OneIndexedArray)
@@ -556,33 +558,6 @@ class ReactionTrajectory(Trajectory):
         self.frames = [new_frame]
         self.save()
 
-    def propagate(self, checkpoint_interval, keep_all=False, time=None):
-        assert isinstance(checkpoint_interval, int) and checkpoint_interval > 0, "interval must be positive integer"
-
-        elapsed_time = 0
-
-        for t in np.arange(self.timestep * len(self.frames), self.stop_time, self.timestep):
-            self.frames.append(self.frames[-1].next(temp=self.frames[-1].bath_temperature, forwards=self.forwards))
-            if (len(self.frames) - 1) % checkpoint_interval == 0:
-                self.save(keep_all=keep_all)
-
-            elapsed_time += self.timestep
-            if time is not None:
-                if elapsed_time >= time:
-                    return
-
-            exit_code = self.termination_function(self.frames[-1])
-            logger.debug(f"termination status:\t{t} fs\t{exit_code} code")
-
-            if exit_code or self.elapsed_since_finished > 0:
-                self.elapsed_since_finished += self.timestep
-                logger.info(f"Reaction trajectory finished! {self.elapsed_since_finished:.1f} fs since termination condition met, {self.time_after_finished:.1f} fs desired.")
-
-            if self.elapsed_since_finished >= self.time_after_finished:
-                self.save(keep_all=keep_all)
-                self.finished = exit_code
-                return
-
 class EquilibrationTrajectory(Trajectory):
     """
     Attributes:
@@ -611,70 +586,60 @@ class EquilibrationTrajectory(Trajectory):
     def __repr__(self):
         return f"EquilibrationTrajectory({len(self.frames)} frames)"
 
-    def initialize(self, positions, **kwargs):
+    def initialize(self, positions, velocities=None, accelerations=None, **kwargs):
         """
         Generates initial frame object for initialization trajectory.
         Velocities are taken from the Maxwell–Boltzmann distribution for the given temperature.
 
         Args:
-            positions (cctk.OneIndexedArray):
+            positions (cctk.OneIndexedArray): starting positions
+            velocities (cctk.OneIndexedArray): starting velocities, optional.
+            accelerations (cctk.OneIndexedArray): starting accelerations, optional.
 
         Returns:
             frame
         """
         logger.info("Initializing new equilibration trajectory...")
         if self.has_checkpoint():
-            self.load_from_checkpoint()
+            self.load_from_checkpoint(slice(-1,None,None))
+            assert len(self.frames) == 1, "didn't load frames properly!"
             return
 
         assert isinstance(positions, cctk.OneIndexedArray), "positions must be a one-indexed array!"
 
-        # move centroid to origin
-        centroid = np.mean(positions, axis=0)
-        positions = positions - centroid
-
         # determine active atoms
         inactive_mask = np.ones(shape=len(positions)).view(cctk.OneIndexedArray)
         inactive_mask[self.active_atoms] = 0
-        inactive_mask  = inactive_mask.astype(bool)
+        inactive_mask = inactive_mask.astype(bool)
 
-        # add random velocity to everything
-        sigma = np.sqrt(self.bath_scheduler(0) * presto.constants.BOLTZMANN_CONSTANT / self.masses.reshape(-1,1))
-        velocities = np.random.normal(scale=sigma, size=positions.shape).view(cctk.OneIndexedArray)
-        velocities[inactive_mask] = 0
-#        velocities = random_gaussian / np.linalg.norm(random_gaussian, axis=1).reshape(-1,1) * sigma
+        if velocities is None:
+            # add random velocity to everything
+            sigma = np.sqrt(self.bath_scheduler(0) * presto.constants.BOLTZMANN_CONSTANT / self.masses.reshape(-1,1))
+            velocities = np.random.normal(scale=sigma, size=positions.shape).view(cctk.OneIndexedArray)
+            velocities[inactive_mask] = 0
 
-        # subtract out center-of-mass translational motion
-        com_translation = np.sum(self.masses.reshape(-1,1) * velocities, axis=0)
-        correction_tran = np.tile(com_translation / np.sum(self.masses[self.active_atoms]), (len(velocities),1))
-        correction_tran[inactive_mask] = 0
-        velocities = velocities - correction_tran
-        com_translation = np.sum(self.masses.reshape(-1,1) * velocities, axis=0)
-        assert np.linalg.norm(np.sum(self.masses.reshape(-1,1) * velocities, axis=0)) < 0.0001, "didn't remove COM translation well enough!"
+            # subtract out center-of-mass translational motion
+            com_translation = np.sum(self.masses.reshape(-1,1) * velocities, axis=0)
+            correction_tran = np.tile(com_translation / np.sum(self.masses[self.active_atoms]), (len(velocities),1))
+            correction_tran[inactive_mask] = 0
+            velocities = velocities - correction_tran
+            com_translation = np.sum(self.masses.reshape(-1,1) * velocities, axis=0)
+            assert np.linalg.norm(np.sum(self.masses.reshape(-1,1) * velocities, axis=0)) < 0.0001, "didn't remove COM translation well enough!"
 
-        velocities = velocities.view(cctk.OneIndexedArray)
-        accelerations = np.zeros_like(positions).view(cctk.OneIndexedArray)
-        self.frames = [presto.frame.Frame(self, positions, velocities, accelerations, self.bath_scheduler(0))]
+            velocities = velocities.view(cctk.OneIndexedArray)
+        else:
+            assert isinstance(velocities, cctk.OneIndexedArray)
+            velocities[inactive_mask] = 0
+
+        if accelerations is None:
+            accelerations = np.zeros_like(positions).view(cctk.OneIndexedArray)
+        else:
+            assert isinstance(accelerations, cctk.OneIndexedArray)
+            accelerations[inactive_mask] = 0
+
+        self.frames = [presto.frame.Frame(self, positions, velocities, accelerations, bath_temperature=self.bath_scheduler(0), time=0.0)]
         self.save()
 
-    def propagate(self, checkpoint_interval, keep_all=False, time=None):
-        assert isinstance(checkpoint_interval, int) and checkpoint_interval > 0, "interval must be positive integer"
-
-        elapsed_time = 0
-        for t in np.arange(self.timestep * self.num_frames(), self.stop_time, self.timestep):
-
-            self.frames.append(self.frames[-1].next(temp=self.bath_scheduler(t)))
-            if (len(self.frames) - 1) % checkpoint_interval == 0:
-                self.save(keep_all=keep_all)
-
-            elapsed_time += self.timestep
-            if time is not None:
-                if elapsed_time >= time:
-                    return
-        self.save(keep_all=keep_all)
-        self.finished = True
-
-
 def join(traj1, traj2):
     """
     Join two reaction trajectories together -- one forward and one reverse!
@@ -717,7 +682,7 @@ def join(traj1, traj2):
     r_frames = copy.deepcopy(traj2.frames)
     r_frames.reverse()
 
-    new_traj.frames = r_frames + f_frames
+    new_traj.frames = r_frames + f_frames[1:] # don't double-count middle frame
 
     if traj1.finished == 2:
         #### if the first traj finished with the reverse condition, reverse order
@@ -725,4 +690,3 @@ def join(traj1, traj2):
 
     return new_traj
 
-
